#include <gnome.h>
#include <liboaf/liboaf.h>
#include <bonobo.h>

/*
 * The Simple Embeddable factory
 */
static BonoboGenericFactory *factory;

/*
 * The count of active Simple Embeddable servers running,
 * when this reaches zero, we shutdown the factory
 */
static int active_servers;

/*
 * This function is called when one Simple Embeddable server is destroyed
 */
static void
embeddable_destroyed (BonoboEmbeddable *embeddable, gpointer user_data)
{
	active_servers--;

	if (active_servers != 0)
		return;
		
	if (factory)
		bonobo_object_unref (BONOBO_OBJECT (factory));
	else
		g_warning ("Serious ref counting error");
	factory = NULL;

	gtk_main_quit ();
}

static BonoboView *
view_factory (BonoboEmbeddable       *embeddable,
	      const Bonobo_ViewFrame  view_frame,
	      void                   *closure)
{
	BonoboView *view;
	GtkWidget  *widget;

	widget = gtk_label_new ("Hello, world!");
	gtk_widget_show_all (widget);

	view = bonobo_view_new (widget);
	/*
	 * The widget is destroyed when the view is destroyed
	 */

	bonobo_view_set_view_frame (view, view_frame);

	return view;
}

/*
 * This function is called when the factory is asked for a new
 * Simple Embeddable server
 */
static BonoboObject *
embeddable_factory (BonoboGenericFactory *this_factory, void *data)
{
	BonoboEmbeddable *embeddable;

	embeddable = bonobo_embeddable_new (view_factory, NULL);

	if (!embeddable)
		return NULL;

	active_servers++;

	/*
	 * Install destructor
	 */
	gtk_signal_connect (GTK_OBJECT (embeddable),
			    "destroy", GTK_SIGNAL_FUNC (embeddable_destroyed),
			    NULL);

	return BONOBO_OBJECT (embeddable);
}

int
main (int argc, char *argv [])
{
	CORBA_ORB orb;

	/*
	 * Init GNOME, oaf and bonobo
	 */

        gnome_init_with_popt_table (
		"com-atena-sample-simple-embeddable", "1.0",
		argc, argv, oaf_popt_options, 0, NULL); 
	orb = oaf_init (argc, argv);

	if (!bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL))
		g_error ("Couldn't initialize bonobo");

	/*
	 * Create and register a Simple Embeddable factory
	 */

	factory = bonobo_generic_factory_new (
		"OAFIID:Com_Atena_Sample_"
		"Simple_Embeddable_EmbeddableFactory",
		embeddable_factory, NULL);

	if (!factory)
		g_error ("Couldn't register a Simple Embeddable factory");

	/*
	 * Let Bonobo start accepting requests
	 */

	bonobo_main ();

	return 0;
}
